<?php

require_once "misc.php";
require_once "db.php";

function generate_rand_int($from = 1000, $to = 9999)
{
    return random_int($from, $to);
}

function generate_otp_code($phone_number)
{
    $id = insertRow([
        "phone_number" => $phone_number,
        "code" => generate_rand_int(),
    ], "otp");

    $row = selectRow("otp", [$id], "id=?");
    $row = $row[0];

    return $row;
}

function getMessageOtpSent($otp)
{
    return "کد تایید : {$otp['code']}\nرپید کد";
}

function get_session_epxire_seconds()
{
    return 86400 * 30;
}

function generate_token($type, $user_id, $phone_number)
{
    $token = generate_password_hash(time() . uniqid("rpd_salt"));
    $id = insertRow([
        "user_id" => $user_id,
        "type" => $type,
        "token" => $token,
        "phone_number" => $phone_number
    ], "session_token");

    return [
        "id" => $id,
        "token" => $token
    ];
}

function getPostHandlers()
{
    return [
        "handler_post_sign_up",
        "handler_post_sign_up_verify",
        "handler_post_sign_up_complete"
    ];
}

function handler_post($post)
{
    $handler = $post['handler'] ?? "";

    $post_handlers = getPostHandlers();

    if (in_array($handler, $post_handlers)) {
        return $handler($post);
    } else {
        die("invalid handler {$handler}");
    }
}

function get_seconds_expire_otp()
{
    return 120;
}

function getResponseHtml()
{
    return [
        "status" => 0, // 0 -> fail , 1 -> success , -1 -> try again
        "message" => "",
        "data" => "",
        "redirect" => "",
    ];
}

function handler_post_sign_up($post)
{
    $phone_number = $post['phone_number'] ?? "";

    $seconds_expire = get_seconds_expire_otp();

    $res = getResponseHtml();

    if (regexCheck($phone_number, '/\d{11}/m')) {
        $old_phone_number = isRowExists("users", [$phone_number], "phone_number=?");
        if ($old_phone_number) {
            $res['message'] = "شماره {$phone_number} قبلا وارد شده";
        } else {

            $limit_message_cbk = function () use (&$limit_seconds, &$res) {
                $limit_seconds = get_seconds_expire_otp();
                $res['message'] = "از قبل کد فعالسازی برای شما فرستاده شده لطفا از زمان ارسال تا {$limit_seconds} ثانیه بعد اقدام کنید";
            };

            // check for sms limitation
            $can_send = limit_send_message(["restrict" => true, "column" => "phone_number", "data" => []], $post['phone_number']);

            if ($can_send) {
                $otp = generate_otp_code($post['phone_number']);
                $sms_res = send_message(getMessageOtpSent($otp), $post['phone_number'], ["column" => "phone_number", "data" => $otp]);

                if (is_int($sms_res)) {
                    $res['status'] = 1;
                    $res['message'] = "کد تایید به شماره {$post['phone_number']} ارسال گردید و در <span data-action=\"lesser\" data-on-end=\"reload\" class=\"timer\">{$seconds_expire}</span> ثانیه دیگر منقضی می گردد";
                    $res['data'] = $otp['id'];
                } else {
                    $limit_message_cbk();
                }
            } else {
                $limit_message_cbk();
            }
        }
    } else {
        $res['message'] = "شماره همراه نا معتبر";
    }


    return $res;
}

function handler_post_sign_up_verify($post)
{
    $res = getResponseHtml();

    $verify_code = $post['verify_code'];
    $id = $post['id'];

    $row = selectRow("otp", [$id], "id=?");

    if (!empty($row[0])) {
        $row = $row[0];
    }

    if (!$row) {
        $res['message'] = "موردی با این مشخصات یافت نشد";
    } else {
        $datetime = getDatetime();
        if (($row['expired_at'] < $datetime)) {
            $res['message'] = "کد وارد شده منقضی شده";
        } else {

            $attempted = $row["attempt"] + 1;
            $dataUpdate = ["attempt" => ($attempted)];

            $expire_otp = function () use (&$dataUpdate) {
                $dataUpdate['expired_at'] = getDatetime((time() - 10));
            };

            if ($row['code'] != $verify_code) {
                $res['message'] = "کاراکتر وارد شده مطابقت ندارد";
                $res['status'] = -1;
                $res['data'] = $id;

                if ($attempted == 3) {
                    $expire_otp();
                }

                $affectedRows = updateRow($dataUpdate, "otp", ["keys" => "id=?", "values" => [$id]]);
            } else if ($row['code'] == $verify_code) {
                $expire_otp();

                $dataUpdate['checked'] = true;
                updateRow($dataUpdate, "otp", ["keys" => "id=?", "values" => [$id]]);

                $token_data = generate_token("auth", -1, $row['phone_number']);
                setcookie("auth", $token_data['token'], (time() + get_session_epxire_seconds()));

                $res['status'] = 1;
                $res['message'] = "موفق , اکنون به صفحه پروفایل وارد می شید";
                $res['redirect'] = "sign.up.complete.php";
            }
        }
    }


    return $res;
}

function handler_post_sign_up_complete($post)
{

    $res = getResponseHtml();

    if (!@$post['username'] || !@$post['fullname']) {
        $res['message'] = "نام کاربری یا نام کامل وارد نشده";
    } else {
        $post['fullname'] = trim(strtolower($post['fullname']));
        $post['fullname'] = preg_replace('/\s{2,}/m' , " " , $post['fullname']);

        if (100 < mb_strlen($post['username']) || 100 < mb_strlen($post['fullname'])) {
            $res['message'] = "نام کاربری یا نام کامل بیشتر از 100 کاراکتر می باشد";
        } else {

            if (!regexCheck($post['username'], '/^[a-zA-Z.]{1,}$/m')) {
                $res['message'] = "نام کاربری معتبر نیست لطفا طبق الگو انتخاب کنید";
            } else {
                $row = isRowExists("users", [$post['username']], "username=?");

                if ($row) {
                    $res['message'] = "نام کاربری {$post['username']} از قبل انتخاب شده نام کاربری دیگری در  نظر بگیرید";
                } else {

                    $user_id = insertRow(["username" => $post['username'], "fullname" => $post['fullname'], "status" => "active", "phone_number" => $GLOBALS['user_data']['phone_number']], "users");
                    updateRow(["user_id" => $user_id], "session_token", ["keys" => "token = ?", "values" => [$_COOKIE['auth']]]);

                    header("Location: view.php");
                }
            }
        }
    }

    return $res;
}
